#include "MatrixRGB.h"
#include "Fonts.h"


/**
* В конструктор дисплея передается 8 номеров ног

Распиновки разъема дисплея
0	G1	GRN   зеленый цвет, последовательность 0-63        g
1	CKL	GRN   тактовый сигнал                              clk
2	STR	GRN   не используется
3	R1	GRN   красный цвет, последовательность 0-63        r
4	R2	GRN   не используется
5	C	GRN   0-й бит адреса строки                        a0
6	B	GRN   1-й бит адреса строки                        a1
7	A	GRN   2-й бит адреса строки                        a2
8	EN	GRN   запись 64-х битов столбцов по каждому цвету  en
9	G2	GRN   синий цвет, последовательность 0-63 бит      b

*/
MatrixRGB::MatrixRGB( uint8_t clk, uint8_t en, uint8_t a0, uint8_t a1, uint8_t a2, uint8_t r, uint8_t g, uint8_t b)
{
  PinCLK = clk;
  PinEN  = en;
  PinA0  = a0;
  PinA1  = a1;
  PinA2  = a2;
  PinR   = r;
  PinG   = g;
  PinB   = b;
}

/**
* Установить цвет одного пиксела RGB
*/
void MatrixRGB::SetColorRGB(boolean red1,boolean green1, boolean blue1)
{
    digitalWrite(PinR,red1);
    digitalWrite(PinG,green1);
    digitalWrite(PinB,blue1);
}

/**
* Установить цвет одного пиксела 
  0 - нет
  1 - blue
  2 - green
  3 - green+blue
  4 - red
  5 - red+blue
  6 - red+green
  7 - red+green+blue
*/
void MatrixRGB::SetColor( byte color )
{
  switch( color )
  {
    case 0: 
       SetColorRGB(0,0,0);
       break;
    case 1: 
       SetColorRGB(0,0,1);
       break;
    case 2: 
       SetColorRGB(0,1,0);
       break;
    case 3: 
       SetColorRGB(0,1,1);
       break;
    case 4: 
       SetColorRGB(1,0,0);
       break;
    case 5: 
       SetColorRGB(1,0,1);
       break;
    case 6: 
       SetColorRGB(1,1,0);
       break;
    case 7: 
       SetColorRGB(1,1,1);
       break;
    }    
}

/**
* Установить строку по трем битам
*/
void MatrixRGB::SetRow3(boolean a0, boolean a1, boolean a2)
{
       digitalWrite(PinA0,a0);
       digitalWrite(PinA1,a1);
       digitalWrite(PinA2,a2);
}  
    
/**
* Установить строку по номеру 0-7
*/
void MatrixRGB::SetRow(int row)
{
    
  switch( row )
  {
    case 0: 
       SetRow3(0,0,0);
       break;
    case 1: 
       SetRow3(0,0,1);
       break;
    case 2: 
       SetRow3(0,1,0);
       break;
    case 3: 
       SetRow3(0,1,1);
       break;
    case 4: 
       SetRow3(1,0,0);
       break;
    case 5: 
       SetRow3(1,0,1);
       break;
    case 6: 
       SetRow3(1,1,0);
       break;
    case 7: 
       SetRow3(1,1,1);
       break;
  }
}

/**
* Сформировать один такт
*/ 
void MatrixRGB::SetClock()
{
    digitalWrite(PinCLK,LOW);
    digitalWrite(PinCLK,HIGH);
}   

/**
* Запись одного символа в буфер
*/
void MatrixRGB::SetPixel(int x, int y, byte c)
{
  if((x < 0) || (x >= 32) || (y < 0) || (y >= 16)) return;
  ScreenBuffer[y][x] = c;

}

/**
* Заливка части буфера одним цветом (если color=0, то очиска буфера
*/
void MatrixRGB::FillColor(byte color,uint8_t x1,uint8_t y1,uint8_t x2, uint8_t y2)
{
  uint8_t xx1=0,yy1=0,xx2=32,yy2=16;
  if( x1>=0 && x1<32 )xx1 = x1;
  if( y1>=0 && y1<16 )yy1 = y1;
  if( x2>=xx1 && x2<32 )xx2 = x2;
  if( y2>=yy1 && y2<16 )yy2 = y2;
  
  for( int i=yy1; i<=yy2; i++ )
  {
     for( int j=xx1; j<=xx2; j++)
     {
       ScreenBuffer[i][j] = color;
     }   
  }
}



/**
* Заливка всего буфера одним цветом (если color=0, то очиска буфера
*/
void MatrixRGB::FillColor(byte color)
{
  for( int i=0; i<16; i++ )
  {
     for( int j=0; j<32; j++)
     {
       ScreenBuffer[i][j] = color;
     }   
  }
}




/**
* Перерисовка экрана С уровнем яркости 0-20
*/
void MatrixRGB::RefreshScreen(unsigned int brightness)
{
   for( int i=0; i<8; i++)
   {
// Формируем значение столбца для первого дисплея
      digitalWrite(PinEN,HIGH);
      SetRow(0);
      for( int j=0; j<32; j++)
      {
        SetColor( ScreenBuffer[i][j]);
        SetClock();
      }
// Формируем значение столбца для второго дисплея
      int i1 = i+8;
      for( int j=0; j<32; j++)
      {
        SetColor( ScreenBuffer[i1][j] );
        SetClock();
      }
// Теперь пишем значение строки
      SetRow(i);
      digitalWrite(PinEN,LOW);
//      delayMicroseconds(100000);
      if( brightness>=0 && brightness<=20)delayMicroseconds(brightness*100);
      else delayMicroseconds(2000);
   }
}



/**
* Инициализация портов дисплея
*/
void MatrixRGB::begin()
{
  pinMode(PinCLK, OUTPUT);
  pinMode(PinEN,  OUTPUT);
  pinMode(PinA0,  OUTPUT);
  pinMode(PinA1,  OUTPUT);
  pinMode(PinA2,  OUTPUT);
  pinMode(PinR,   OUTPUT);
  pinMode(PinG,   OUTPUT);
  pinMode(PinB,   OUTPUT);
}


/**
* Напечатать одну цифру шрифтом 3x6
*/
void MatrixRGB::PrintChar6h(int x, int y, byte ch, byte color)
{
  if( ch < 0 || ch > 18 )ch = 18;
  for( int i=0; i<6; i++){
    byte c = pgm_read_byte(&font3x6[ch][i]);
    if( c & B100 )SetPixel(x,y+i,color);
    else SetPixel(x,y+i,0);
    if( c & B010 )SetPixel(x+1,y+i,color);
    else SetPixel(x+1,y+i,0);
    if( c & B001 )SetPixel(x+2,y+i,color);
    else SetPixel(x+2,y+i,0);
  }
}

/**
* Напечатать одну цифру шрифтом 4x8
*/
void MatrixRGB::PrintChar8h(int x, int y, byte ch, byte color)
{
  if( ch < 0 || ch > 10 )ch = 10;
  for( int i=0; i<8; i++){
    byte c = pgm_read_byte(&font4x8[ch][i]);
    if( c & B1000 )SetPixel(x,y+i,color);
    else SetPixel(x,y+i,0);
    if( c & B0100 )SetPixel(x+1,y+i,color);
    else SetPixel(x+1,y+i,0);
    if( c & B0010 )SetPixel(x+2,y+i,color);
    else SetPixel(x+2,y+i,0);
    if( c & B0001 )SetPixel(x+3,y+i,color);
    else SetPixel(x+3,y+i,0);
  }
}


/**
* Нарисовать иконку 5x5
*/
void MatrixRGB::PrintIcon5x5(int x, int y,byte ch, byte color)
{
  if( ch < 0 || ch > 5 )return;
  for( uint8_t i=0; i<5; i++){
    uint8_t c = pgm_read_byte(&icons5x5[ch][i]);
    if( c & B10000 )SetPixel(x,y+i,color);
    if( c & B01000 )SetPixel(x+1,y+i,color);
    if( c & B00100 )SetPixel(x+2,y+i,color);
    if( c & B00010 )SetPixel(x+3,y+i,color);
    if( c & B00001 )SetPixel(x+4,y+i,color);
  }
}


/**
* Нарисовать иконку 7x7
*/
void MatrixRGB::PrintIcon7x7(int x, int y,byte ch, byte color)
{
  if( ch < 0 || ch > 8 )return;
  for( uint8_t i=0; i<7; i++){
    uint8_t c = pgm_read_byte(&icons7x7[ch][i]);
    if( c & B1000000 )SetPixel(x,y+i,color);
    if( c & B0100000 )SetPixel(x+1,y+i,color);
    if( c & B0010000 )SetPixel(x+2,y+i,color);
    if( c & B0001000 )SetPixel(x+3,y+i,color);
    if( c & B0000100 )SetPixel(x+4,y+i,color);
    if( c & B0000010 )SetPixel(x+5,y+i,color);
    if( c & B0000001 )SetPixel(x+6,y+i,color);
  }
}

/**
* Нарисовать иконку 9x9
*/
void MatrixRGB::PrintIcon9x9(int x, int y,byte ch, byte color)
{
  if( ch < 0 || ch > 3 )return;
  for( uint8_t i=0; i<9; i++){
    uint16_t c = pgm_read_word(&icons9x9[ch][i]);
    if( c & 0x0100 )SetPixel(x,y+i,color);
    if( c & 0x0080 )SetPixel(x+1,y+i,color);
    if( c & 0x0040 )SetPixel(x+2,y+i,color);
    if( c & 0x0020 )SetPixel(x+3,y+i,color);
    if( c & 0x0010 )SetPixel(x+4,y+i,color);
    if( c & 0x0008 )SetPixel(x+5,y+i,color);
    if( c & 0x0004 )SetPixel(x+6,y+i,color);
    if( c & 0x0002 )SetPixel(x+7,y+i,color);
    if( c & 0x0001 )SetPixel(x+8,y+i,color);
  }
}


